import * as helpers from "./modules/helpers.js";

const navigation = {
  home: {
    label: "Home",
    path: "/",
    command: "home",
    depth: 0,
    submenu: {
      menu: {
        label: "Menu",
        path: "/menu",
        command: "menu",
        depth: 1,
        submenu: {
          call: {
            label: "Call",
            path: "/menu/call",
            command: "call",
            depth: 2,
            submenu: {
              emergency: {
                label: "Emergency",
                command: "emergency",
                path: "/menu/call/emergency",
                depth: 3,
                message: "Calling Emergency",
              },
              contact1: {
                label: "Flute Inc.",
                command: "contact A",
                path: "/menu/call/contact1",
                depth: 3,
                message: "Calling Flute Inc.",
              },
              contact2: {
                label: "Brays AG",
                command: "contact B",
                path: "/menu/call/contact2",
                depth: 3,
                message: "Calling Brays AG",
              },
            },
          },
          check: {
            label: "Check",
            path: "/menu/check",
            command: "check",
            depth: 2,
            submenu: {
              daily: {
                label: "Daily",
                command: "daily",
                path: "/menu/check/daily",
                depth: 3,
                message: "Displaying Daily Checklist",
              },
              weekly: {
                label: "Weekly",
                command: "weekly",
                path: "/menu/check/weekly",
                depth: 3,
                message: "Displaying Weekly Checklist",
              },
              monthly: {
                label: "Monthly",
                command: "monthly",
                path: "/menu/check/monthly",
                depth: 3,
                message: "Displaying Monthly Checklist",
              },
            },
          },
          apps: {
            label: "Apps",
            path: "/menu/apps",
            command: "apps",
            depth: 2,
            submenu: {
              record: {
                label: "Record",
                command: "record",
                path: "/menu/apps/record",
                depth: 3,
                message: "Opening Recording App",
              },
              breakout: {
                label: "Breakout",
                command: "breakout",
                path: "/menu/apps/breakout",
                depth: 3,
                message: "Opening Breakout App",
              },
              statistics: {
                label: "Statistics",
                command: "statistics",
                path: "/menu/apps/statistics",
                depth: 3,
                message: "Opening Statistics App",
              },
            },
          },
        },
      },
    },
  },
};

// init application
function init() {
  // build navigation
  const appEl = document.getElementById("app");
  const navigationEl = document.createElement("nav");
  const listEl = document.createElement("ul");
  navigationEl.id = "navigation";
  navigationEl.appendChild(listEl);
  appEl.appendChild(navigationEl);

  // navigate to current path or home
  const pathname = window.location.pathname;
  if (pathname) navigateToPath(pathname);
  else navigateToPath("/");

  // Add a popstate event listener to the window object
  window.addEventListener("popstate", function (event) {
    var state = event.state;

    // Load the appropriate page based on the state object
    // without pushing it to the history
    if (state) navigateToPath(state.pageUrl, false);
    else navigateToPath("/", false);
  });

  // adding controls
  addMouseControls();
  addButtonControls();
}

// update Navigation
function updateNavigation(object) {
  // update navigation
  const listEl = document.createElement("ul");

  // add navigation items
  for (let key in object.submenu) {
    let itemEl = buildNavigationItemEl(object.submenu[key]);
    itemEl.classList.add("forward-btn");
    listEl.appendChild(itemEl);
  }

  // add "special back" button
  if (object.depth > 0) {
    let backItemEl;
    if (object.depth === 1)
      backItemEl = buildNavigationItemEl({
        label: "Close",
        path: "/",
        command: "close",
      });
    else if (object.depth === 2)
      backItemEl = buildNavigationItemEl({
        label: "Back",
        path: "/menu",
        command: "back",
      });
    else if (object.depth === 3)
      backItemEl = buildNavigationItemEl({
        label: "Exit",
        path: "/",
        command: "exit",
      });

    backItemEl.classList.add("back-btn");
    listEl.appendChild(backItemEl);
  }

  const navigationEl = document.getElementById("navigation");
  navigationEl.replaceChildren(listEl);

  focusNavigationItem(1);

  // build navigation item
  function buildNavigationItemEl(item) {
    const itemEl = document.createElement("li");
    itemEl.className = "link";
    const linkEl = document.createElement("a");
    linkEl.href = item.path;
    linkEl.innerHTML = item.label;
    linkEl.dataset.command = item.command;
    itemEl.appendChild(linkEl);

    if (item.label.toLowerCase() !== item.command.toLowerCase()) {
      const commandEl = document.createElement("div");
      commandEl.className = "command";
      commandEl.innerHTML = item.command;
      itemEl.appendChild(commandEl);
    }

    return itemEl;
  }
}

// General Navigation
// ===================
// navigate to path
function navigateToPath(path, pushState = true) {
  // find object with path
  let navObj = helpers.findNestedObject(navigation, "path", path);
  if (!navObj) navObj = navigation.home; // fallback to home
  updateNavigation(navObj);

  // State Object
  var stateObj = {
    pageTitle: navObj.label,
    pageUrl: navObj.path,
  };

  // Push state to browser history
  if (pushState)
    history.pushState(stateObj, stateObj.pageTitle, stateObj.pageUrl);
}

// navigate to currently selected item
function navigateToSelected() {
  const currentItem = document.querySelector(
    "#navigation > ul > li[data-selected] a"
  );
  const path = currentItem.getAttribute("href");
  navigateToPath(path);
}

// focus specific navigation item
function focusNavigationItem(n) {
  // unselect all items
  const navigationList = document.querySelectorAll("#navigation li");
  for (let item of navigationList) delete item.dataset.selected;

  // select requested element
  const navigationItem = document.querySelector(
    `#navigation > ul > li:nth-child(${n})`
  );
  navigationItem.dataset.selected = "";
}

// focus next navigation item
function focusNextNavigationItem() {
  // get currently selected element
  const currentItem = document.querySelector(
    "#navigation > ul > li[data-selected]"
  );
  if (currentItem.nextSibling) {
    delete currentItem.dataset.selected;
    currentItem.nextSibling.dataset.selected = "";
  } else focusNavigationItem(1);
}

// focus previous navigation item
function focusPreviousNavigationItem() {
  // get currently selected element
  let currentItem = document.querySelector(
    "#navigation > ul > li[data-selected]"
  );
  if (currentItem.previousSibling) {
    delete currentItem.dataset.selected;
    currentItem.previousSibling.dataset.selected = "";
  } else {
    let childCount = document.querySelector("#navigation > ul").children.length;
    focusNavigationItem(childCount);
  }
}

// moving down navigation tree
function moveUpNavigationLevel() {
  const path = window.location.pathname;
  let parentPath = path.slice(0, path.lastIndexOf("/"));
  navigateToPath(parentPath);
}

// moving down navigation tree
function moveDownNavigationLevel() {
  const currentItem = document.querySelector(
    "#navigation > ul > li[data-selected]"
  );
  const isBackButton = currentItem.classList.contains("back-btn");
  if (!isBackButton) navigateToSelected();
}

// Add Mouse Click Controls
// ==================
function addMouseControls() {
  const appEl = document.getElementById("app");
  appEl.addEventListener("click", onMouseClick);

  // on menu item mouseclick
  function onMouseClick(event) {
    event.preventDefault();
    const listItem = event.target.closest("li.link");
    if (listItem) {
      const link = listItem.querySelector("a");
      const path = link.getAttribute("href");
      navigateToPath(path);
    }
  }
}

// Add Button Controls
// ===================
function addButtonControls() {
  // add two button controls for menu
  document.addEventListener("keydown", onKeyDown);

  // on keydown event
  function onKeyDown(event) {
    // prevent defaults
    if (
      event.code === "Tab" ||
      event.code === "Enter" ||
      event.code === "Escape" ||
      event.code === "ArrowDown" ||
      event.code === "ArrowUp" ||
      event.code === "ArrowRight"
    )
      event.preventDefault();

    // browsing navigation items
    if (event.code === "Tab" || event.code === "ArrowDown")
      focusNextNavigationItem();
    else if (event.code === "ArrowUp") focusPreviousNavigationItem();
    else if (event.code === "Enter") navigateToSelected();
    else if (event.code === "ArrowRight") moveDownNavigationLevel();
    else if (event.code === "ArrowLeft" || event.code === "Escape")
      moveUpNavigationLevel();
  }

  // listening for hardware buttons
  // navigator.mediaSession.setActionHandler('volumeup', function() {
  //   // Handle volume up event
  // });

  // navigator.mediaSession.setActionHandler('volumedown', function() {
  //   // Handle volume down event
  // });
}

init();
